@@ -51,8 +51,7 @@
 #' @export
 
 
-as.data.frame.meta <- function(x, row.names = NULL,
-                               optional = FALSE, ...) {
+as.data.frame.meta <- function(x, row.names = NULL, optional = FALSE, ...) {
   
   
   ##
@@ -72,6 +71,10 @@ as.data.frame.meta <- function(x, row.names = NULL,
   ##
   x$call <- NULL
   
+  ## Remove data set from output
+  ##
+  x$data <- NULL
+  
   if (!is.null(x$approx.TE) && all(x$approx.TE == ""))
     x$approx.TE <- NULL
   ##
