@@ -37,8 +37,6 @@
 #'   confidence limits.
 #' @param level The level used to calculate confidence intervals for
 #'   individual studies.
-#' @param level.ma The level used to calculate confidence intervals
-#'   for meta-analysis estimates.
 #' @param common A logical indicating whether a common effect
 #'   meta-analysis should be conducted.
 #' @param random A logical indicating whether a random effects
@@ -53,29 +51,35 @@
 #'   level.
 #' @param prediction A logical indicating whether a prediction
 #'   interval should be printed.
-#' @param level.predict The level used to calculate prediction
-#'   interval for a new study.
-#' @param hakn A logical indicating whether the method by Hartung and
-#'   Knapp should be used to adjust test statistics and confidence
-#'   intervals.
-#' @param adhoc.hakn A character string indicating whether an \emph{ad
-#'   hoc} variance correction should be applied in the case of an
-#'   arbitrarily small Hartung-Knapp variance estimate, see Details.
 #' @param method.tau A character string indicating which method is
 #'   used to estimate the between-study variance \eqn{\tau^2} and its
-#'   square root \eqn{\tau}. Either \code{"DL"}, \code{"PM"},
-#'   \code{"REML"}, \code{"ML"}, \code{"HS"}, \code{"SJ"},
-#'   \code{"HE"}, or \code{"EB"}, can be abbreviated.
+#'   square root \eqn{\tau} (see \code{\link{meta-package}}).
 #' @param method.tau.ci A character string indicating which method is
 #'   used to estimate the confidence interval of \eqn{\tau^2} and
-#'   \eqn{\tau}. Either \code{"QP"}, \code{"BJ"}, \code{"J"},
-#'   \code{"PL"}, or \code{""}, can be abbreviated.
+#'   \eqn{\tau} (see \code{\link{meta-package}}).
 #' @param tau.preset Prespecified value for the square root of the
 #'   between-study variance \eqn{\tau^2}.
 #' @param TE.tau Overall treatment effect used to estimate the
 #'   between-study variance tau-squared.
 #' @param tau.common A logical indicating whether tau-squared should
 #'   be the same across subgroups.
+#' @param level.ma The level used to calculate confidence intervals
+#'   for meta-analysis estimates.
+#' @param method.random.ci A character string indicating which method
+#'   is used to calculate confidence interval and test statistic for
+#'   random effects estimate (see \code{\link{meta-package}}).
+#' @param adhoc.hakn.ci A character string indicating whether an
+#'   \emph{ad hoc} variance correction should be applied in the case
+#'   of an arbitrarily small Hartung-Knapp variance estimate (see
+#'   \code{\link{meta-package}}).
+#' @param level.predict The level used to calculate prediction
+#'   interval for a new study.
+#' @param method.predict A character string indicating which method is
+#'   used to calculate a prediction interval (see
+#'   \code{\link{meta-package}}).
+#' @param adhoc.hakn.pi A character string indicating whether an
+#'   \emph{ad hoc} variance correction should be applied for
+#'   prediction interval (see \code{\link{meta-package}}).
 #' @param null.effect A numeric value specifying the effect under the
 #'   null hypothesis.
 #' @param method.bias A character string indicating which test is to
@@ -118,6 +122,8 @@
 #' @param prediction.subgroup A logical indicating whether prediction
 #'   intervals should be printed for subgroups.
 #' @param byvar Deprecated argument (replaced by 'subgroup').
+#' @param hakn Deprecated argument (replaced by 'method.random.ci').
+#' @param adhoc.hakn Deprecated argument (replaced by 'adhoc.hakn.ci').
 #' @param keepdata A logical indicating whether original data (set)
 #'   should be kept in meta object.
 #' @param warn A logical indicating whether the addition of
@@ -160,6 +166,13 @@
 #' \code{\link[metafor]{rma.glmm}} function from R package
 #' \bold{metafor}.
 #' 
+#' A three-level random effects meta-analysis model (Van den Noortgate
+#' et al., 2013) is utilized if argument \code{cluster} is used and at
+#' least one cluster provides more than one estimate. Internally,
+#' \code{\link[metafor]{rma.mv}} is called to conduct the analysis and
+#' \code{\link[metafor]{weights.rma.mv}} with argument \code{type =
+#' "rowsum"} is used to calculate random effects weights.
+#' 
 #' Default settings are utilised for several arguments (assignments
 #' using \code{\link{gs}} function). These defaults can be changed for
 #' the current R session using the \code{\link{settings.meta}}
@@ -192,104 +205,6 @@
 #' For the Freeman-Tukey (Freeman & Tukey, 1950) and square root
 #' transformation as well as GLMMs no continuity correction is used.
 #' }
-#' 
-#' \subsection{Estimation of between-study variance}{
-#' 
-#' The following methods to estimate the between-study variance
-#' \eqn{\tau^2} are available for the inverse variance method:
-#' \itemize{
-#' \item DerSimonian-Laird estimator (\code{method.tau = "DL"})
-#' \item Paule-Mandel estimator (\code{method.tau = "PM"})
-#' \item Restricted maximum-likelihood estimator (\code{method.tau =
-#'   "REML"})
-#' \item Maximum-likelihood estimator (\code{method.tau = "ML"})
-#' \item Hunter-Schmidt estimator (\code{method.tau = "HS"})
-#' \item Sidik-Jonkman estimator (\code{method.tau = "SJ"})
-#' \item Hedges estimator (\code{method.tau = "HE"})
-#' \item Empirical Bayes estimator (\code{method.tau = "EB"})
-#' }
-#' See \code{\link{metagen}} for more information on these
-#' estimators. Note, the maximum-likelihood method is utilized for
-#' GLMMs.
-#' }
-#' 
-#' \subsection{Confidence interval for the between-study variance}{
-#'
-#' The following methods to calculate a confidence interval for
-#' \eqn{\tau^2} and \eqn{\tau} are available.
-#' \tabular{ll}{
-#' \bold{Argument}\tab \bold{Method} \cr
-#' \code{method.tau.ci = "J"}\tab Method by Jackson (2013) \cr
-#' \code{method.tau.ci = "BJ"}\tab Method by Biggerstaff and Jackson (2008) \cr
-#' \code{method.tau.ci = "QP"}\tab Q-Profile method (Viechtbauer, 2007) \cr
-#' \code{method.tau.ci = "PL"}\tab Profile-Likelihood method for
-#'   three-level meta-analysis \cr
-#'  \tab (Van den Noortgate et al., 2013) \cr
-#' \code{method.tau.ci = ""}\tab No confidence interval
-#' }
-#' 
-#' See \code{\link{metagen}} for more information on these methods.
-#'
-#' For GLMMs, no confidence intervals for \eqn{\tau^2} and \eqn{\tau}
-#' are calculated.
-#' }
-#' 
-#' \subsection{Hartung-Knapp method}{
-#' 
-#' Hartung and Knapp (2001a,b) proposed an alternative method for
-#' random effects meta-analysis based on a refined variance estimator
-#' for the treatment estimate. Simulation studies (Hartung and Knapp,
-#' 2001a,b; IntHout et al., 2014; Langan et al., 2019) show improved
-#' coverage probabilities compared to the classic random effects
-#' method.
-#'
-#' In rare settings with very homogeneous treatment estimates, the
-#' Hartung-Knapp variance estimate can be arbitrarily small resulting
-#' in a very narrow confidence interval (Knapp and Hartung, 2003;
-#' Wiksten et al., 2016). In such cases, an \emph{ad hoc} variance
-#' correction has been proposed by utilising the variance estimate
-#' from the classic random effects model with the HK method (Knapp and
-#' Hartung, 2003; IQWiQ, 2020). An alternative approach is to use the
-#' wider confidence interval of classic common or random effects
-#' meta-analysis and the HK method (Wiksten et al., 2016; Jackson et
-#' al., 2017).
-#'
-#' Argument \code{adhoc.hakn} can be used to choose the \emph{ad hoc}
-#' method:
-#' \tabular{ll}{
-#' \bold{Argument}\tab \bold{\emph{Ad hoc} method} \cr
-#' \code{adhoc.hakn = ""}\tab not used \cr
-#' \code{adhoc.hakn = "se"}\tab use variance correction if HK standard
-#'  error is smaller \cr
-#'  \tab than standard error from classic random effects
-#'  \cr
-#'  \tab meta-analysis (Knapp and Hartung, 2003) \cr
-#' \code{adhoc.hakn = "iqwig6"}\tab use variance correction if HK
-#'  confidence interval \cr
-#'  \tab is narrower than CI from classic random effects model \cr
-#'  \tab with DerSimonian-Laird estimator (IQWiG, 2020) \cr
-#' \code{adhoc.hakn = "ci"}\tab use wider confidence interval of
-#'  classic random effects \cr
-#'  \tab and HK meta-analysis \cr
-#'  \tab (Hybrid method 2 in Jackson et al., 2017)
-#' }
-#' 
-#' For GLMMs, a method similar to Knapp and Hartung (2003) is
-#' implemented, see description of argument \code{tdist} in
-#' \code{\link[metafor]{rma.glmm}}, and the \emph{ad hoc} variance
-#' correction is not available.
-#' }
-#' 
-#' \subsection{Prediction interval}{
-#' 
-#' A prediction interval for the proportion in a new study (Higgins et
-#' al., 2009) is calculated if arguments \code{prediction} and
-#' \code{random} are \code{TRUE}. Note, the definition of
-#' prediction intervals varies in the literature. This function
-#' implements equation (12) of Higgins et al., (2009) which proposed a
-#' \emph{t} distribution with \emph{K-2} degrees of freedom where
-#' \emph{K} corresponds to the number of studies in the meta-analysis.
-#' }
 #'
 #' \subsection{Subgroup analysis}{
 #' 
@@ -344,175 +259,20 @@
 #' specify the time unit used in individual studies (default:
 #' "person-years"). This information is printed in summaries and
 #' forest plots if argument \code{irscale} is not equal to 1.
+#'
+#' A prediction interval will only be shown if \code{prediction =
+#' TRUE}.
 #' }
 #' 
 #' @return
 #' An object of class \code{c("metarate", "meta")} with corresponding
-#' \code{print}, \code{summary}, and \code{forest} functions. The
-#' object is a list containing the following components:
-#' \item{event, n, studlab, exclude, cluster,}{As defined above.}
-#' \item{sm, incr, method.incr, method.ci,}{As defined above.}
-#' \item{level, level.ma,}{As defined above.}
-#' \item{common, random,}{As defined above.}
-#' \item{overall, overall.hetstat,}{As defined above.}
-#' \item{hakn, adhoc.hakn, method.tau, method.tau.ci,}{As defined above.}
-#' \item{tau.preset, TE.tau, null.hypothesis,}{As defined above.}
-#' \item{method.bias, tau.common, title, complab, outclab,}{As defined
-#'   above.}
-#' \item{subgroup, subgroup.name, print.subgroup.name, sep.subgroup, warn}{As defined
-#'   above.}
-#' \item{TE, seTE}{Estimated (un)transformed incidence rate and its
-#'   standard error for individual studies.}
-#' \item{lower, upper}{Lower and upper confidence interval limits for
-#'   individual studies.}
-#' \item{zval, pval}{z-value and p-value for test of treatment effect
-#'   for individual studies.}
-#' \item{w.common, w.random}{Weight of individual studies (in common
-#'   effect and random effects model).}
-#' \item{TE.common, seTE.common}{Estimated overall (un)transformed
-#'   incidence rate and standard error (common effect model).}
-#' \item{lower.common, upper.common}{Lower and upper confidence interval
-#'   limits (common effect model).}
-#' \item{statistic.common, pval.common}{z-value and p-value for test of
-#'   overall effect (common effect model).}
-#' \item{TE.random, seTE.random}{Estimated overall (un)transformed
-#'   incidence rate and standard error (random effects model).}
-#' \item{lower.random, upper.random}{Lower and upper confidence
-#'   interval limits (random effects model).}
-#' \item{statistic.random, pval.random}{z-value or t-value and
-#'   corresponding p-value for test of overall effect (random effects
-#'   model).}
-#' \item{prediction, level.predict}{As defined above.}
-#' \item{seTE.predict}{Standard error utilised for prediction
-#'   interval.}
-#' \item{lower.predict, upper.predict}{Lower and upper limits of
-#'   prediction interval.}
-#' \item{k}{Number of estimates combined in meta-analysis.}
-#' \item{k.study}{Number of studies combined in meta-analysis.}
-#' \item{k.all}{Number of all studies.}
-#' \item{k.TE}{Number of studies with estimable effects.}
-#' \item{Q}{Heterogeneity statistic Q.}
-#' \item{df.Q}{Degrees of freedom for heterogeneity statistic.}
-#' \item{pval.Q}{P-value of heterogeneity test.}
-#' \item{Q.LRT}{Heterogeneity statistic for likelihood-ratio test
-#'   (only if \code{method = "GLMM"}).}
-#' \item{df.Q.LRT}{Degrees of freedom for likelihood-ratio test}
-#' \item{pval.Q.LRT}{P-value of likelihood-ratio test.}
-#' \item{tau2}{Between-study variance \eqn{\tau^2}.}
-#' \item{se.tau2}{Standard error of \eqn{\tau^2}.}
-#' \item{lower.tau2, upper.tau2}{Lower and upper limit of confidence
-#'   interval for \eqn{\tau^2}.}
-#' \item{tau}{Square-root of between-study variance \eqn{\tau}.}
-#' \item{lower.tau, upper.tau}{Lower and upper limit of confidence
-#'   interval for \eqn{\tau}.}
-#' \item{H}{Heterogeneity statistic H.}
-#' \item{lower.H, upper.H}{Lower and upper confidence limit for
-#'  heterogeneity statistic H.}
-#' \item{I2}{Heterogeneity statistic I\eqn{^2}.}
-#' \item{lower.I2, upper.I2}{Lower and upper confidence limit for
-#'   heterogeneity statistic I\eqn{^2}.}
-#' \item{Rb}{Heterogeneity statistic R\eqn{_b}.}
-#' \item{lower.Rb, upper.Rb}{Lower and upper confidence limit for
-#'   heterogeneity statistic R\eqn{_b}.}
-#' \item{method}{A character string indicating method used for
-#'   pooling: \code{"Inverse"}}
-#' \item{df.hakn}{Degrees of freedom for test of treatment effect for
-#'   Hartung-Knapp method (only if \code{hakn = TRUE}).}
-#' \item{bylevs}{Levels of grouping variable - if \code{subgroup} is not
-#'   missing.}
-#' \item{TE.common.w, seTE.common.w}{Estimated treatment effect and
-#'   standard error in subgroups (common effect model) - if
-#'   \code{subgroup} is not missing.}
-#' \item{lower.common.w, upper.common.w}{Lower and upper confidence
-#'   interval limits in subgroups (common effect model) - if
-#'   \code{subgroup} is not missing.}
-#' \item{statistic.common.w, pval.common.w}{z-value and p-value for test
-#'   of treatment effect in subgroups (common effect model) - if
-#'   \code{subgroup} is not missing.}
-#' \item{TE.random.w, seTE.random.w}{Estimated treatment effect and
-#'   standard error in subgroups (random effects model) - if
-#'   \code{subgroup} is not missing.}
-#' \item{lower.random.w, upper.random.w}{Lower and upper confidence
-#'   interval limits in subgroups (random effects model) - if
-#'   \code{subgroup} is not missing.}
-#' \item{statistic.random.w, pval.random.w}{z-value or t-value and
-#'   corresponding p-value for test of treatment effect in subgroups
-#'   (random effects model) - if \code{subgroup} is not missing.}
-#' \item{w.common.w, w.random.w}{Weight of subgroups (in common effect
-#'   and random effects model) - if \code{subgroup} is not missing.}
-#' \item{df.hakn.w}{Degrees of freedom for test of treatment effect
-#'   for Hartung-Knapp method in subgroups - if \code{subgroup} is not
-#'   missing and \code{hakn = TRUE}.}
-#' \item{n.harmonic.mean.w}{Harmonic mean of number of observations in
-#'   subgroups (for back transformation of Freeman-Tukey Double
-#'   arcsine transformation) - if \code{subgroup} is not missing.}
-#' \item{event.w}{Number of events in subgroups - if \code{subgroup} is
-#'   not missing.}
-#' \item{n.w}{Number of observations in subgroups - if \code{subgroup} is
-#'   not missing.}
-#' \item{k.w}{Number of studies combined within subgroups - if
-#'   \code{subgroup} is not missing.}
-#' \item{k.all.w}{Number of all studies in subgroups - if \code{subgroup}
-#'   is not missing.}
-#' \item{Q.w.common}{Overall within subgroups heterogeneity statistic Q
-#'   (based on common effect model) - if \code{subgroup} is not missing.}
-#' \item{Q.w.random}{Overall within subgroups heterogeneity statistic
-#'   Q (based on random effects model) - if \code{subgroup} is not
-#'   missing (only calculated if argument \code{tau.common} is TRUE).}
-#' \item{df.Q.w}{Degrees of freedom for test of overall within
-#'   subgroups heterogeneity - if \code{subgroup} is not missing.}
-#' \item{pval.Q.w.common}{P-value of within subgroups heterogeneity
-#'   statistic Q (based on common effect model) - if \code{subgroup} is
-#'   not missing.}
-#' \item{pval.Q.w.random}{P-value of within subgroups heterogeneity
-#'   statistic Q (based on random effects model) - if \code{subgroup} is
-#'   not missing.}
-#' \item{Q.b.common}{Overall between subgroups heterogeneity statistic
-#'   Q (based on common effect model) - if \code{subgroup} is not
-#'   missing.}
-#' \item{Q.b.random}{Overall between subgroups heterogeneity statistic
-#'   Q (based on random effects model) - if \code{subgroup} is not
-#'   missing.}
-#' \item{df.Q.b}{Degrees of freedom for test of overall between
-#'   subgroups heterogeneity - if \code{subgroup} is not missing.}
-#' \item{pval.Q.b.common}{P-value of between subgroups heterogeneity
-#'   statistic Q (based on common effect model) - if \code{subgroup} is
-#'   not missing.}
-#' \item{pval.Q.b.random}{P-value of between subgroups heterogeneity
-#'   statistic Q (based on random effects model) - if \code{subgroup} is
-#'   not missing.}
-#' \item{tau.w}{Square-root of between-study variance within subgroups
-#'   - if \code{subgroup} is not missing.}
-#' \item{H.w}{Heterogeneity statistic H within subgroups - if
-#'   \code{subgroup} is not missing.}
-#' \item{lower.H.w, upper.H.w}{Lower and upper confidence limit for
-#'   heterogeneity statistic H within subgroups - if \code{subgroup} is
-#'   not missing.}
-#' \item{I2.w}{Heterogeneity statistic I\eqn{^2} within subgroups - if
-#'   \code{subgroup} is not missing.}
-#' \item{lower.I2.w, upper.I2.w}{Lower and upper confidence limit for
-#'   heterogeneity statistic I\eqn{^2} within subgroups - if \code{subgroup} is
-#'   not missing.}
-#' \item{incr.event}{Increment added to number of events.}
-#' \item{keepdata}{As defined above.}
-#' \item{data}{Original data (set) used in function call (if
-#'   \code{keepdata = TRUE}).}
-#' \item{subset}{Information on subset of original data used in
-#'   meta-analysis (if \code{keepdata = TRUE}).}
-#' \item{.glmm.common}{GLMM object generated by call of
-#'   \code{\link[metafor]{rma.glmm}} function (common effect model).}
-#' \item{.glmm.random}{GLMM object generated by call of
-#'   \code{\link[metafor]{rma.glmm}} function (random effects model).}
-#' \item{call}{Function call.}
-#' \item{version}{Version of R package \bold{meta} used to create
-#'   object.}
-#' \item{version.metafor}{Version of R package \bold{metafor} used for
-#'   GLMMs.}
+#' generic functions (see \code{\link{meta-object}}).
 #' 
 #' @author Guido Schwarzer \email{sc@@imbi.uni-freiburg.de}
 #' 
-#' @seealso \code{\link{update.meta}}, \code{\link{metacont}},
-#'   \code{\link{metagen}}, \code{\link{print.meta}}
+#' @seealso \code{\link{meta-package}}, \code{\link{update.meta}},
+#'   \code{\link{metacont}}, \code{\link{metagen}},
+#'   \code{\link{print.meta}}
 #' 
 #' @references
 #' Borenstein M, Hedges LV, Higgins JP, Rothstein HR (2010):
@@ -526,47 +286,6 @@
 #' \emph{Annals of Mathematical Statistics},
 #' \bold{21}, 607--11
 #' 
-#' Higgins JPT, Thompson SG, Spiegelhalter DJ (2009):
-#' A re-evaluation of random-effects meta-analysis.
-#' \emph{Journal of the Royal Statistical Society: Series A},
-#' \bold{172}, 137--59
-#' 
-#' Hartung J, Knapp G (2001a):
-#' On tests of the overall treatment effect in meta-analysis with
-#' normally distributed responses.
-#' \emph{Statistics in Medicine},
-#' \bold{20}, 1771--82
-#' 
-#' Hartung J, Knapp G (2001b):
-#' A refined method for the meta-analysis of controlled clinical
-#' trials with binary outcome.
-#' \emph{Statistics in Medicine},
-#' \bold{20}, 3875--89
-#'
-#' IntHout J, Ioannidis JPA, Borm GF (2014):
-#' The Hartung-Knapp-Sidik-Jonkman method for random effects
-#' meta-analysis is straightforward and considerably outperforms the
-#' standard DerSimonian-Laird method.
-#' \emph{BMC Medical Research Methodology},
-#' \bold{14}, 25
-#' 
-#' IQWiG (2020):
-#' General Methods: Version 6.0.
-#' \url{https://www.iqwig.de/en/about-us/methods/methods-paper/}
-#'
-#' Jackson D, Law M, Rücker G, Schwarzer G (2017): 
-#' The Hartung-Knapp modification for random-effects meta-analysis: A
-#' useful refinement but are there any residual concerns?
-#' \emph{Statistics in Medicine},
-#' \bold{36}, 3923--34
-#' 
-#' Langan D, Higgins JPT, Jackson D, Bowden J, Veroniki AA,
-#' Kontopantelis E, et al. (2019):
-#' A comparison of heterogeneity variance estimators in simulated
-#' random-effects meta-analyses.
-#' \emph{Research Synthesis Methods},
-#' \bold{10}, 83--98
-#' 
 #' Stijnen T, Hamza TH, Ozdemir P (2010):
 #' Random effects meta-analysis of event outcome in the framework of
 #' the generalized linear mixed model with applications in sparse
@@ -583,12 +302,6 @@
 #' Conducting Meta-Analyses in R with the Metafor Package.
 #' \emph{Journal of Statistical Software},
 #' \bold{36}, 1--48
-#'
-#' Wiksten A, Rücker G, Schwarzer G (2016):
-#' Hartung-Knapp method is not always conservative compared with
-#' fixed-effect meta-analysis.
-#' \emph{Statistics in Medicine},
-#' \bold{35}, 2503--15
 #' 
 #' @examples
 #' # Apply various meta-analysis methods to estimate incidence rates
@@ -630,21 +343,26 @@ metarate <- function(event, time, studlab,
                      ##
                      incr = gs("incr"), method.incr = gs("method.incr"),
                      ##
-                     method.ci = gs("method.ci.rate"),
-                     level = gs("level"), level.ma = gs("level.ma"),
+                     method.ci = gs("method.ci.rate"), level = gs("level"),
+                     ##
                      common = gs("common"),
                      random = gs("random") | !is.null(tau.preset),
                      overall = common | random,
                      overall.hetstat = common | random,
+                     prediction = gs("prediction") | !missing(method.predict),
                      ##
-                     hakn = gs("hakn"), adhoc.hakn = gs("adhoc.hakn"),
                      method.tau,
                      method.tau.ci = gs("method.tau.ci"),
                      tau.preset = NULL, TE.tau = NULL,
                      tau.common = gs("tau.common"),
                      ##
-                     prediction = gs("prediction"),
+                     level.ma = gs("level.ma"),
+                     method.random.ci = gs("method.random.ci"),
+                     adhoc.hakn.ci = gs("adhoc.hakn.ci"),
+                     ##
                      level.predict = gs("level.predict"),
+                     method.predict = gs("method.predict"),
+                     adhoc.hakn.pi = gs("adhoc.hakn.pi"),
                      ##
                      null.effect = NA,
                      ##
@@ -667,7 +385,7 @@ metarate <- function(event, time, studlab,
                      sep.subgroup = gs("sep.subgroup"),
                      test.subgroup = gs("test.subgroup"),
                      prediction.subgroup = gs("prediction.subgroup"),
-                     byvar,
+                     byvar, hakn, adhoc.hakn,
                      ##
                      keepdata = gs("keepdata"),
                      warn = gs("warn"), warn.deprecated = gs("warn.deprecated"),
@@ -688,20 +406,35 @@ metarate <- function(event, time, studlab,
   ##
   chklevel(level)
   ##
-  chklogical(hakn)
-  missing.adhoc.hakn <- missing(adhoc.hakn)
-  adhoc.hakn <- setchar(adhoc.hakn, gs("adhoc4hakn"))
-  if (missing(method.tau))
+  missing.method.tau <- missing(method.tau)
+  if (missing.method.tau)
     method.tau <- if (method == "GLMM") "ML" else gs("method.tau")
   method.tau <- setchar(method.tau, gs("meth4tau"))
+  ##
   if (is.null(method.tau.ci))
     method.tau.ci <- if (method.tau == "DL") "J" else "QP"
   method.tau.ci <- setchar(method.tau.ci, gs("meth4tau.ci"))
+  tau.common <- replaceNULL(tau.common, FALSE)
   chklogical(tau.common)
   ##
   chklogical(prediction)
   chklevel(level.predict)
   ##
+  missing.method.predict <- missing(method.predict)
+  ##
+  method.tau <-
+    setmethodtau(method.tau, missing.method.tau,
+                 method.predict, missing.method.predict)
+  method.predict <-
+    setmethodpredict(method.predict, missing.method.predict,
+                     method.tau, missing.method.tau)
+  ##
+  if (method.predict == "NNF")
+    is.installed.package("pimeta", argument = "method.predict", value = "NNF")
+  ##
+  missing.adhoc.hakn.pi <- missing(adhoc.hakn.pi)
+  adhoc.hakn.pi <- setchar(adhoc.hakn.pi, gs("adhoc4hakn.pi"))
+  ##
   chknumeric(null.effect, length = 1)
   ##
   method.bias <- setmethodbias(method.bias)
@@ -761,6 +494,23 @@ metarate <- function(event, time, studlab,
                        warn.deprecated)
   chklogical(random)
   ##
+  method.random.ci <-
+    deprecated2(method.random.ci, missing(method.random.ci),
+                hakn, missing(hakn),
+                warn.deprecated)
+  if (is.logical(method.random.ci))
+    if (method.random.ci)
+      method.random.ci <- "HK"
+    else
+      method.random.ci <- "classic"
+  method.random.ci <- setchar(method.random.ci, gs("meth4random.ci"))
+  ##
+  missing.adhoc.hakn.ci <- missing(adhoc.hakn.ci)
+  adhoc.hakn.ci <-
+    deprecated2(adhoc.hakn.ci, missing(adhoc.hakn.ci),
+                adhoc.hakn, missing(adhoc.hakn), warn.deprecated)
+  adhoc.hakn.ci <- setchar(adhoc.hakn.ci, gs("adhoc4hakn.ci"))
+  ##
   missing.subgroup.name <- missing(subgroup.name)
   subgroup.name <-
     deprecated(subgroup.name, missing.subgroup.name, args, "bylab",
@@ -769,7 +519,8 @@ metarate <- function(event, time, studlab,
   print.subgroup.name <-
     deprecated(print.subgroup.name, missing(print.subgroup.name),
                args, "print.byvar", warn.deprecated)
-  print.subgroup.name <- replaceNULL(print.subgroup.name, FALSE)
+  print.subgroup.name <-
+    replaceNULL(print.subgroup.name, gs("print.subgroup.name"))
   chklogical(print.subgroup.name)
   ##
   sep.subgroup <-
@@ -1168,23 +919,49 @@ metarate <- function(event, time, studlab,
     stop("Generalised linear mixed models only possible with ",
          "argument 'method.tau = \"ML\"'.")
   ##
-  if (is.glmm & hakn & adhoc.hakn != "") {
-    if (!missing.adhoc.hakn)
-      warning("Ad hoc variance correction for Hartung-Knapp method ",
-              "not available for GLMMs.",
+  if (is.glmm & method.random.ci == "KR")
+    stop("Kenward-Roger method for random effects meta-analysis not ",
+         "available for GLMMs.",
+         call. = FALSE)
+  ##
+  if (is.glmm & method.predict == "KR")
+    stop("Kenward-Roger method for prediction interval not ",
+         "available for GLMMs.",
+         call. = FALSE)
+  ##
+  if (is.glmm & method.predict == "NNF")
+    stop("Bootstrap method for prediction interval not ",
+         "available for GLMMs.",
+         call. = FALSE)
+  ##
+  if (is.glmm & method.random.ci == "HK" & adhoc.hakn.ci != "") {
+    if (!missing.adhoc.hakn.ci)
+      warning("Ad hoc correction for Hartung-Knapp method not ",
+              "available for GLMMs.",
               call. = FALSE)
-    adhoc.hakn <- ""
+    adhoc.hakn.ci <- ""
+  }
+  ##
+  if (is.glmm & method.predict == "HK" & adhoc.hakn.pi != "") {
+    if (!missing.adhoc.hakn.pi)
+      warning("Ad hoc Hartung-Knapp correction ffor prediction interval not ",
+              "available for GLMMs.",
+              call. = FALSE)
+    adhoc.hakn.pi <- ""
   }
   ##
   if (is.glmm & sparse)
     if ((!missing(incr) & any(incr != 0)) |
         allincr | addincr)
-      warning("Note, for method = \"GLMM\", continuity correction only used to calculate individual study results.")
+      warning("Note, for method = \"GLMM\", continuity correction only ",
+              "used to calculate individual study results.",
+              call. = FALSE)
   ##
   if (is.glmm) {
     if (!is.null(TE.tau)) {
       if (warn)
-        warning("Argument 'TE.tau' not considered for GLMM.")
+        warning("Argument 'TE.tau' not considered for GLMM.",
+                call. = FALSE)
       TE.tau <- NULL
     }
     ##
@@ -1203,14 +980,15 @@ metarate <- function(event, time, studlab,
   ##
   k <- sum(!is.na(event[!exclude]) & !is.na(time[!exclude]))
   ##
-  if (k == 1 & hakn)
-    hakn <- FALSE
+  if (k == 1 & method.random.ci != "classic")
+    method.random.ci <- "classic"
   ##
   if (is.glmm & k > 0) {
     glmm.common <-
       runNN(rma.glmm,
             list(xi = event[!exclude], ti = time[!exclude],
-                 method = "FE", test = ifelse(hakn, "t", "z"),
+                 method = "FE",
+                 test = ifelse(method.random.ci == "HK", "t", "z"),
                  level = 100 * level.ma,
                  measure = "IRLN", control = control,
                  ...))
@@ -1227,20 +1005,25 @@ metarate <- function(event, time, studlab,
                ##
                sm = sm,
                level = level,
-               level.ma = level.ma,
+               ##
                common = common,
                random = random,
                overall = overall,
                overall.hetstat = overall.hetstat,
+               prediction = prediction,
                ##
-               hakn = hakn, adhoc.hakn = adhoc.hakn,
                method.tau = method.tau, method.tau.ci = method.tau.ci,
                tau.preset = tau.preset,
                TE.tau = TE.tau,
                tau.common = FALSE,
                ##
-               prediction = prediction,
+               level.ma = level.ma,
+               method.random.ci = method.random.ci,
+               adhoc.hakn.ci = adhoc.hakn.ci,
+               ##
                level.predict = level.predict,
+               method.predict = method.predict,
+               adhoc.hakn.pi = adhoc.hakn.pi,
                ##
                null.effect = transf.null.effect,
                ##
@@ -1276,7 +1059,6 @@ metarate <- function(event, time, studlab,
               incr = if (length(unique(incr)) == 1) unique(incr) else incr,
               method.incr = method.incr,
               sparse = sparse,
-              allincr = allincr, addincr = addincr,
               method.ci = method.ci,
               incr.event = incr.event)
   ##
@@ -1285,6 +1067,14 @@ metarate <- function(event, time, studlab,
   ##
   m$n.e <- NULL
   m$n.c <- NULL
+  m$pscale <- NULL
+  m$irscale <- NULL
+  m$irunit <- NULL
+  m$method.ci <- NULL
+  m$method.mean <- NULL
+  m$approx.TE <- NULL
+  m$approx.seTE <- NULL
+  ##
   m$label.e <- ""
   m$label.c <- ""
   m$label.left <- ""
@@ -1323,7 +1113,7 @@ metarate <- function(event, time, studlab,
         runNN(rma.glmm,
               list(xi = event[!exclude], ti = time[!exclude],
                    method = method.tau,
-                   test = ifelse(hakn, "t", "z"),
+                   test = ifelse(method.random.ci == "HK", "t", "z"),
                    level = 100 * level.ma,
                    measure = "IRLN", control = control,
                    ...))
@@ -1339,7 +1129,8 @@ metarate <- function(event, time, studlab,
     seTE.random <- as.numeric(glmm.random$se)
     ##
     ci.r <- ci(TE.random, seTE.random, level = level.ma,
-               null.effect = transf.null.effect, df = if (hakn) k - 1)
+               null.effect = transf.null.effect,
+               df = if (method.random.ci == "HK") k - 1)
     ##
     res$w.random <- rep(NA, length(event))
     ##
@@ -1353,26 +1144,28 @@ metarate <- function(event, time, studlab,
     ##
     ## Prediction interval
     ##
-    if (k >= 3) {
-      tau2.calc <- if (is.na(glmm.random$tau2)) 0 else glmm.random$tau2
-      seTE.predict <- sqrt(seTE.random^2 + tau2.calc)
-      ci.p <- ci(TE.random, seTE.predict, level.predict, k - 2)
-      res$seTE.predict <- seTE.predict
-      res$lower.predict <- ci.p$lower
-      res$upper.predict <- ci.p$upper
-    }
-    else {
-      res$seTE.predict <- NA
-      res$lower.predict <- NA
-      res$upper.predict <- NA
-    }
+    res$upper.predict <- res$lower.predict <- res$seTE.predict <- NA
     ##
-    res$Q <- glmm.random$QE.Wld
+    tau2.calc <- if (is.na(glmm.random$tau2)) 0 else glmm.random$tau2
+    seTE.predict <- sqrt(seTE.random^2 + tau2.calc)
+    ##
+    if (method.predict == "HTS" && k >= 3)
+      ci.p <- ci(TE.random, seTE.predict, level.predict, k - 2)
+    else if (method.predict == "S")
+      ci.p <- ci(TE.random, seTE.predict, level.predict)
+    else
+      ci.p <- list(lower = NA, upper = NA)
+    ##
+    res$seTE.predict <- seTE.predict
+    res$lower.predict <- ci.p$lower
+    res$upper.predict <- ci.p$upper
+    ##
+    res$Q <- if (glmm.random$k > 1) glmm.random$QE.Wld else 0
     res$df.Q <- glmm.random$QE.df
     res$pval.Q <- pvalQ(res$Q, res$df.Q)
     ##
-    res$Q.LRT      <- glmm.random$QE.LRT
-    res$df.Q.LRT   <- res$df.Q
+    res$Q.LRT <- if (glmm.random$k > 1) glmm.random$QE.LRT else 0
+    res$df.Q.LRT <- res$df.Q
     res$pval.Q.LRT <- pvalQ(res$Q.LRT, res$df.Q.LRT)
     ##
     if (k > 1) {
@@ -1412,8 +1205,8 @@ metarate <- function(event, time, studlab,
     res$version.metafor <- packageDescription("metafor")$Version
     ##
     if (by) {
-      n.by <- length(unique(subgroup[!exclude]))
-      if (n.by > 1)
+      n.subgroups <- length(unique(subgroup[!exclude]))
+      if (n.subgroups > 1)
         subgroup.glmm <- factor(subgroup[!exclude], bylevs(subgroup[!exclude]))
       else
         subgroup.glmm <- NA
@@ -1424,9 +1217,10 @@ metarate <- function(event, time, studlab,
                 list(xi = event[!exclude],
                      ti = time[!exclude],
                      mods =
-                       if (n.by > 1) as.call(~ subgroup.glmm) else NULL,
+                       if (n.subgroups > 1)
+                         as.call(~ subgroup.glmm) else NULL,
                      method = method.tau,
-                     test = ifelse(hakn, "t", "z"),
+                     test = ifelse(method.random.ci == "HK", "t", "z"),
                      level = 100 * level.ma,
                      measure = "IRLN", control = control,
                      data = data.frame(subgroup.glmm),
@@ -1443,9 +1237,10 @@ metarate <- function(event, time, studlab,
                     list(xi = event[!exclude],
                          ti = time[!exclude],
                          mods =
-                           if (n.by > 1) as.call(~ subgroup.glmm) else NULL,
+                           if (n.subgroups > 1)
+                             as.call(~ subgroup.glmm) else NULL,
                          method = "FE",
-                         test = ifelse(hakn, "t", "z"),
+                         test = ifelse(method.random.ci == "HK", "t", "z"),
                          level = 100 * level.ma,
                          measure = "IRLN", control = control,
                          data = data.frame(subgroup.glmm),
@@ -1492,6 +1287,8 @@ metarate <- function(event, time, studlab,
   res$irunit  <- irunit
   ##
   res$call <- match.call()
+  res$allincr <- allincr
+  res$addincr <- addincr
   ##
   if (keepdata) {
     res$data <- data
@@ -1514,11 +1311,8 @@ metarate <- function(event, time, studlab,
     ##
     if (!tau.common) {
       res <- c(res, subgroup(res))
-      if (res$three.level) {
-        res$Q.b.random <- NA
-        res$df.Q.b <- NA
-        res$pval.Q.b.random <- NA
-      }
+      if (res$three.level)
+        res <- setNA3(res)
     }
     else if (!is.null(tau.preset))
       res <- c(res, subgroup(res, tau.preset))
@@ -1534,15 +1328,7 @@ metarate <- function(event, time, studlab,
         res <- c(res, subgroup(res, hcc$tau.resid))
     }
     ##
-    if (!tau.common || !is.null(tau.preset)) {
-      res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
-      res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
-      ##
-      res$Q.resid <- res$df.Q.resid <- res$pval.Q.resid <- NA
-      res$H.resid <- res$lower.H.resid <- res$upper.H.resid <- NA
-      res$I2.resid <- res$lower.I2.resid <- res$upper.I2.resid <- NA
-    }
-    else {
+    if (tau.common && is.null(tau.preset)) {
       res$Q.w.random <- hcc$Q.resid
       res$df.Q.w.random <- hcc$df.Q.resid
       res$pval.Q.w.random <- hcc$pval.Q.resid
